/*! rollup-plugin-htmlpipe v1.0.0 | MIT Licensed | allex <allex.wxn@gmail.com> (http://iallex.com/) */
import { readFile, writeFile } from 'fs';
import { basename, dirname, join } from 'path';

/**
 * Takes an HTML file as a template then adds the bundle to the final file.
 * @param {Object} options The options object.
 * @return {Object} The rollup code object.
 */
var index = ((options = {}) => {
  const {
    template,
    target
  } = options;

  // Get the target file name.
  let targetName = basename(target || template);

  // Add the file suffix if it isn't there.
  const targetFile = targetName.indexOf('.html') < 0 ? `${targetName}.html` : targetName;

  return {
    name: 'htmlpipe',
    ongenerate(outputOptions, bundle) {
      const file = outputOptions.file;
      return promisify(readFile, template, 'utf8').then(buffer => {
        // Convert buffer to a string and get the </body> index.
        const tmpl = buffer.toString('utf8');
        const bodyCloseTag = tmpl.lastIndexOf('</body>');

        const scripts = options.inline ? `<script>\n${bundle.code}</script>` : `<script src="${basename(file)}"></script>`;

        // Inject the script tag before the body close tag.
        const injected = [tmpl.slice(0, bodyCloseTag), scripts, tmpl.slice(bodyCloseTag, tmpl.length)].join('');

        // Write the injected template to a file.
        return promisify(writeFile, join(dirname(file), targetFile), injected).then(() => {});
      });
    }
  };
});

/**
 * Promisify's a function.
 * @param {Function} fn The function to turn into a promise.
 * @param {any[]} args The arguments for the function.
 * @return {Promise} A Promise for the function.
 */
function promisify(fn, ...args) {
  return new Promise((resolve, reject) => {
    try {
      fn(...args, (err, res) => {
        if (err) reject(err);else resolve(res);
      });
    } catch (e) {
      reject(e);
    }
  });
}

export default index;
